@@ -24,7 +24,7 @@ VALID = re.compile(r"^[a-zA-Z_]\w*$")
 
 def is_keyword(identifier):
     try:
-        RESERVED.parseString(identifier)
+        RESERVED.parse_string(identifier)
         return True
     except Exception:
         return False
@@ -65,7 +65,7 @@ def escape(ident, ansi_quotes, should_quote):
 
 def Operator(_op):
     op_prec = precedence[binary_ops[_op]]
-    op = " {0} ".format(_op).upper()
+    op = " {0} ".format(_op).replace("_", " ").upper()
 
     def func(self, json, prec):
         acc = []
@@ -75,9 +75,12 @@ def Operator(_op):
             k, v = first(json.items())
             json = [k, {"literal": v}]
 
-        for v in listwrap(json):
-            acc.append(self.dispatch(v, op_prec))
-        if prec > op_prec:
+        for i, v in enumerate(listwrap(json)):
+            if i == 0:
+                acc.append(self.dispatch(v, op_prec + 1))
+            else:
+                acc.append(self.dispatch(v, op_prec))
+        if prec >= op_prec:
             return op.join(acc)
         else:
             return f"({op.join(acc)})"
@@ -105,6 +108,8 @@ def isolate(expr, sql, prec):
 
 unordered_clauses = [
     "with",
+    "distinct_on",
+    "select_distinct",
     "select",
     "from",
     "where",
@@ -140,9 +145,9 @@ class Formatter:
     _binary_and = Operator("&")
     _binary_or = Operator("|")
     _like = Operator("like")
-    _not_like = Operator("not_like")
+    _not_like = Operator("not like")
     _rlike = Operator("rlike")
-    _not_rlike = Operator("not_rlike")
+    _not_rlike = Operator("not rlike")
     _union = Operator("union")
     _union_all = Operator("union all")
     _intersect = Operator("intersect")
@@ -181,17 +186,73 @@ class Formatter:
 
         return text(json)
 
-    def sql_list(self, json, prec=precedence['from'] - 1):
+    def sql_list(self, json, prec=precedence["from"] - 1):
         sql = ", ".join(self.dispatch(element, prec=MAX_PRECEDENCE) for element in json)
-        if prec >= precedence['from']:
+        if prec >= precedence["from"]:
             return sql
         else:
             return f"({sql})"
 
-    def value(self, json, prec=precedence['from']):
+    def value(self, json, prec=precedence["from"]):
         parts = [self.dispatch(json["value"], prec)]
+        if "over" in json:
+            over = json["over"]
+            parts.append("OVER")
+            window = []
+            if "partitionby" in over:
+                window.append("PARTITION BY")
+                window.append(self.dispatch(over["partitionby"]))
+            if "orderby" in over:
+                window.append(self.orderby(over, precedence["window"]))
+            if "range" in over:
+
+                def wordy(v):
+                    if v < 0:
+                        return [text(abs(v)), "PRECEDING"]
+                    elif v > 0:
+                        return [text(v), "FOLLOWING"]
+
+                window.append("ROWS")
+                range = over["range"]
+                min = range.get("min")
+                max = range.get("max")
+
+                if min is None:
+                    if max is None:
+                        window.pop()  # not expected, but deal
+                    elif max == 0:
+                        window.append("UNBOUNDED PRECEDING")
+                    else:
+                        window.append("BETWEEN")
+                        window.append("UNBOUNDED PRECEDING")
+                        window.append("AND")
+                        window.extend(wordy(max))
+                elif min == 0:
+                    if max is None:
+                        window.append("UNBOUNDED FOLLOWING")
+                    elif max == 0:
+                        window.append("CURRENT ROW")
+                    else:
+                        window.extend(wordy(max))
+                else:
+                    if max is None:
+                        window.append("BETWEEN")
+                        window.extend(wordy(min))
+                        window.append("AND")
+                        window.append("UNBOUNDED FOLLOWING")
+                    elif max == 0:
+                        window.extend(wordy(min))
+                    else:
+                        window.append("BETWEEN")
+                        window.extend(wordy(min))
+                        window.append("AND")
+                        window.extend(wordy(max))
+
+            window = " ".join(window)
+            parts.append(f"({window})")
         if "name" in json:
             parts.extend(["AS", self.dispatch(json["name"])])
+
         return " ".join(parts)
 
     def op(self, json, prec):
@@ -210,7 +271,7 @@ class Formatter:
         # treat as regular function call
         if isinstance(value, dict) and len(value) == 0:
             return (
-                    key.upper() + "()"
+                key.upper() + "()"
             )  # NOT SURE IF AN EMPTY dict SHOULD BE DELT WITH HERE, OR IN self.format()
         else:
             params = ", ".join(self.dispatch(p) for p in listwrap(value))
@@ -220,13 +281,15 @@ class Formatter:
         return "~{0}".format(self.dispatch(value))
 
     def _exists(self, value, prec):
-        return "{0} IS NOT NULL".format(self.dispatch(value, precedence['is']))
+        return "{0} IS NOT NULL".format(self.dispatch(value, precedence["is"]))
 
     def _missing(self, value, prec):
-        return "{0} IS NULL".format(self.dispatch(value, precedence['is']))
+        return "{0} IS NULL".format(self.dispatch(value, precedence["is"]))
 
     def _collate(self, pair, prec):
-        return "{0} COLLATE {1}".format(self.dispatch(pair[0], precedence['collate']), pair[1])
+        return "{0} COLLATE {1}".format(
+            self.dispatch(pair[0], precedence["collate"]), pair[1]
+        )
 
     def _case(self, checks, prec):
         parts = ["CASE"]
@@ -244,7 +307,9 @@ class Formatter:
 
     def _literal(self, json, prec):
         if isinstance(json, list):
-            return "({0})".format(", ".join(self._literal(v, precedence['literal']) for v in json))
+            return "({0})".format(", ".join(
+                self._literal(v, precedence["literal"]) for v in json
+            ))
         elif isinstance(json, string_types):
             return "'{0}'".format(json.replace("'", "''"))
         else:
@@ -252,18 +317,30 @@ class Formatter:
 
     def _between(self, json, prec):
         return "{0} BETWEEN {1} AND {2}".format(
-            self.dispatch(json[0], precedence['between']), self.dispatch(json[1], precedence['between']),
-            self.dispatch(json[2], precedence['between'])
+            self.dispatch(json[0], precedence["between"]),
+            self.dispatch(json[1], precedence["between"]),
+            self.dispatch(json[2], precedence["between"]),
         )
 
     def _not_between(self, json, prec):
         return "{0} NOT BETWEEN {1} AND {2}".format(
-            self.dispatch(json[0], precedence['between']), self.dispatch(json[1], precedence['between']),
-            self.dispatch(json[2], precedence['between'])
+            self.dispatch(json[0], precedence["between"]),
+            self.dispatch(json[1], precedence["between"]),
+            self.dispatch(json[2], precedence["between"]),
         )
 
     def _distinct(self, json, prec):
-        return "DISTINCT " + ", ".join(self.dispatch(v) for v in listwrap(json))
+        return "DISTINCT " + ", ".join(
+            self.dispatch(v, precedence["select"]) for v in listwrap(json)
+        )
+
+    def _select_distinct(self, json, prec):
+        return "SELECT DISTINCT " + ", ".join(self.dispatch(v) for v in listwrap(json))
+
+    def _distinct_on(self, json, prec):
+        return (
+            "DISTINCT ON (" + ", ".join(self.dispatch(v) for v in listwrap(json)) + ")"
+        )
 
     def _join_on(self, json, prec):
         detected_join = join_keywords & set(json.keys())
@@ -292,20 +369,20 @@ class Formatter:
     def ordered_query(self, json, prec):
         if json.keys() & set(unordered_clauses) - {"from"}:
             # regular query
-            acc = [self.unordered_query(json, precedence['order'])]
+            acc = [self.unordered_query(json, precedence["order"])]
         else:
             # set-op expression
-            acc = [self.dispatch(json['from'], precedence['order'])]
+            acc = [self.dispatch(json["from"], precedence["order"])]
 
         acc.extend(
             part
             for clause in ordered_clauses
             if clause in json
-            for part in [getattr(self, clause)(json, precedence['order'])]
+            for part in [getattr(self, clause)(json, precedence["order"])]
             if part
         )
         sql = " ".join(acc)
-        if prec >= precedence['order']:
+        if prec >= precedence["order"]:
             return sql
         else:
             return f"({sql})"
@@ -315,10 +392,10 @@ class Formatter:
             part
             for clause in unordered_clauses
             if clause in json
-            for part in [getattr(self, clause)(json, precedence['from'])]
+            for part in [getattr(self, clause)(json, precedence["from"])]
             if part
         )
-        if prec >= precedence['from']:
+        if prec >= precedence["from"]:
             return sql
         else:
             return f"({sql})"
@@ -339,14 +416,24 @@ class Formatter:
         if "top" in json:
             top = self.dispatch(json["top"])
             return f"SELECT TOP ({top}) {param}"
+        if "distinct_on" in json:
+            return param
         else:
             return f"SELECT {param}"
 
+    def distinct_on(self, json, prec):
+        param = ", ".join(self.dispatch(s) for s in listwrap(json["distinct_on"]))
+        return f"SELECT DISTINCT ON ({param})"
+
+    def select_distinct(self, json, prec):
+        param = ", ".join(self.dispatch(s) for s in listwrap(json["select_distinct"]))
+        return f"SELECT DISTINCT {param}"
+
     def from_(self, json, prec):
         is_join = False
         from_ = json["from"]
         if isinstance(from_, dict) and is_set_op & from_.keys():
-            return self.op(from_, precedence['from'])
+            return self.op(from_, precedence["from"])
 
         from_ = listwrap(from_)
         parts = []
@@ -372,7 +459,11 @@ class Formatter:
 
     def orderby(self, json, prec):
         param = ", ".join(
-            (self.dispatch(s["value"], precedence["order"]) + " " + s.get("sort", "").upper()).strip()
+            (
+                self.dispatch(s["value"], precedence["order"])
+                + " "
+                + s.get("sort", "").upper()
+            ).strip()
             for s in listwrap(json["orderby"])
         )
         return f"ORDER BY {param}"
