@@ -1,9 +1,20 @@
-from mo_sql_parsing.utils import *
+# encoding: utf-8
+#
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this file,
+# You can obtain one at http://mozilla.org/MPL/2.0/.
+#
+# Contact: Kyle Lahnakoski (kyle@lahnakoski.com)
+#
 
 # SQL CONSTANTS
-NULL = Keyword("null", caseless=True).addParseAction(lambda: SQL_NULL)
-TRUE = Keyword("true", caseless=True).addParseAction(lambda: True)
-FALSE = Keyword("false", caseless=True).addParseAction(lambda: False)
+from mo_parsing import *
+
+from mo_sql_parsing.utils import SQL_NULL, int_num, to_json_call, ansi_string, ansi_ident
+
+NULL = Keyword("null", caseless=True) / (lambda: SQL_NULL)
+TRUE = Keyword("true", caseless=True) / (lambda: True)
+FALSE = Keyword("false", caseless=True) / (lambda: False)
 NOCASE = Keyword("nocase", caseless=True)
 ASC = Keyword("asc", caseless=True)
 DESC = Keyword("desc", caseless=True)
@@ -60,7 +71,6 @@ PRIMARY_KEY = Group(PRIMARY + KEY).set_parser_name("primary_key")
 FOREIGN_KEY = Group(FOREIGN + KEY).set_parser_name("foreign_key")
 
 # SIMPLE OPERATORS
-CASTING = Literal("::").set_parser_name("concat")
 CONCAT = Literal("||").set_parser_name("concat")
 MUL = Literal("*").set_parser_name("mul")
 DIV = Literal("/").set_parser_name("div")
@@ -125,8 +135,8 @@ IS_NOT = Group(IS + NOT).set_parser_name("is_not")
 
 _SIMILAR = Keyword("similar", caseless=True)
 _TO = Keyword("to", caseless=True)
-SIMILAR_TO = Group(_SIMILAR + _TO).set_parser_name("is_not")
-NOT_SIMILAR_TO = Group(_SIMILAR + _TO).set_parser_name("is_not")
+SIMILAR_TO = Group(_SIMILAR + _TO).set_parser_name("similar_to")
+NOT_SIMILAR_TO = Group(NOT + _SIMILAR + _TO).set_parser_name("not_similar_to")
 
 RESERVED = MatchFirst([
     # ONY INCLUDE SINGLE WORDS
@@ -137,7 +147,6 @@ RESERVED = MatchFirst([
     BETWEEN,
     BY,
     CASE,
-    CAST,
     COLLATE,
     CONSTRAINT,
     CREATE,
@@ -209,8 +218,6 @@ join_keywords = {
     "left outer join",
 }
 
-unary_ops = (NEG, NOT, BINARY_NOT)
-
 precedence = {
     # https://www.sqlite.org/lang_expr.html
     "literal": -1,
@@ -218,11 +225,11 @@ precedence = {
     "collate": 0,
     "concat": 1,
     "mul": 2,
-    "div": 2,
+    "div": 1.5,
     "mod": 2,
     "neg": 3,
     "add": 3,
-    "sub": 3,
+    "sub": 2.5,
     "binary_not": 4,
     "binary_and": 4,
     "binary_or": 4,
@@ -246,8 +253,9 @@ precedence = {
     "not_similar_to": 8,
     "and": 10,
     "or": 11,
-    "select":  30,
+    "select": 30,
     "from": 30,
+    "window": 35,
     "union": 40,
     "union_all": 40,
     "except": 40,
@@ -258,7 +266,6 @@ precedence = {
 
 
 KNOWN_OPS = [
-    CASTING,
     COLLATE,
     CONCAT,
     MUL | DIV | MOD,
@@ -356,46 +363,40 @@ durations = {
     "epoch": "epoch",
 }
 
-_size = Optional(LB + intNum("params") + RB)
-_sizes = Optional(LB + intNum("params") + "," + intNum("params") + RB)
+_size = Optional(LB + int_num("params") + RB)
+_sizes = Optional(LB + int_num("params") + "," + int_num("params") + RB)
 
 # KNOWN TYPES
-ARRAY = Group(Keyword("array", caseless=True)("op")).addParseAction(to_json_call)
-BIGINT = Group(Keyword("bigint", caseless=True)("op")).addParseAction(to_json_call)
-BOOL = Group(Keyword("bool", caseless=True)("op")).addParseAction(to_json_call)
-BOOLEAN = Group(Keyword("boolean", caseless=True)("op")).addParseAction(to_json_call)
-DOUBLE = Group(Keyword("double", caseless=True)("op")).addParseAction(to_json_call)
-FLOAT64 = Group(Keyword("float64", caseless=True)("op")).addParseAction(to_json_call)
-FLOAT = Group(Keyword("float", caseless=True)("op")).addParseAction(to_json_call)
-GEOMETRY = Group(Keyword("geometry", caseless=True)("op")).addParseAction(to_json_call)
-INTEGER = Group(Keyword("integer", caseless=True)("op")).addParseAction(to_json_call)
-INT = Group(Keyword("int", caseless=True)("op")).addParseAction(to_json_call)
-INT32 = Group(Keyword("int32", caseless=True)("op")).addParseAction(to_json_call)
-INT64 = Group(Keyword("int64", caseless=True)("op")).addParseAction(to_json_call)
-REAL = Group(Keyword("real", caseless=True)("op")).addParseAction(to_json_call)
-TEXT = Group(Keyword("text", caseless=True)("op")).addParseAction(to_json_call)
-SMALLINT = Group(Keyword("smallint", caseless=True)("op")).addParseAction(to_json_call)
-STRING = Group(Keyword("string", caseless=True)("op")).addParseAction(to_json_call)
-STRUCT = Group(Keyword("struct", caseless=True)("op")).addParseAction(to_json_call)
+ARRAY = Group(Keyword("array", caseless=True)("op")) / to_json_call
+BIGINT = Group(Keyword("bigint", caseless=True)("op")) / to_json_call
+BOOL = Group(Keyword("bool", caseless=True)("op")) / to_json_call
+BOOLEAN = Group(Keyword("boolean", caseless=True)("op")) / to_json_call
+DOUBLE = Group(Keyword("double", caseless=True)("op")) / to_json_call
+FLOAT64 = Group(Keyword("float64", caseless=True)("op")) / to_json_call
+FLOAT = Group(Keyword("float", caseless=True)("op")) / to_json_call
+GEOMETRY = Group(Keyword("geometry", caseless=True)("op")) / to_json_call
+INTEGER = Group(Keyword("integer", caseless=True)("op")) / to_json_call
+INT = Group(Keyword("int", caseless=True)("op")) / to_json_call
+INT32 = Group(Keyword("int32", caseless=True)("op")) / to_json_call
+INT64 = Group(Keyword("int64", caseless=True)("op")) / to_json_call
+REAL = Group(Keyword("real", caseless=True)("op")) / to_json_call
+TEXT = Group(Keyword("text", caseless=True)("op")) / to_json_call
+SMALLINT = Group(Keyword("smallint", caseless=True)("op")) / to_json_call
+STRING = Group(Keyword("string", caseless=True)("op")) / to_json_call
+STRUCT = Group(Keyword("struct", caseless=True)("op")) / to_json_call
 
-BLOB = (Keyword("blob", caseless=True)("op") + _size).addParseAction(to_json_call)
-BYTES = (Keyword("bytes", caseless=True)("op") + _size).addParseAction(to_json_call)
-CHAR = (Keyword("char", caseless=True)("op") + _size).addParseAction(to_json_call)
-VARCHAR = (Keyword("varchar", caseless=True)("op") + _size).addParseAction(to_json_call)
-VARBINARY = (
-    Keyword("varbinary", caseless=True)("op") + _size
-).addParseAction(to_json_call)
-TINYINT = (Keyword("tinyint", caseless=True)("op") + _size).addParseAction(to_json_call)
+BLOB = (Keyword("blob", caseless=True)("op") + _size) / to_json_call
+BYTES = (Keyword("bytes", caseless=True)("op") + _size) / to_json_call
+CHAR = (Keyword("char", caseless=True)("op") + _size) / to_json_call
+VARCHAR = (Keyword("varchar", caseless=True)("op") + _size) / to_json_call
+VARBINARY = (Keyword("varbinary", caseless=True)("op") + _size) / to_json_call
+TINYINT = (Keyword("tinyint", caseless=True)("op") + _size) / to_json_call
 
-DECIMAL = (
-    Keyword("decimal", caseless=True)("op") + _sizes
-).addParseAction(to_json_call)
+DECIMAL = (Keyword("decimal", caseless=True)("op") + _sizes) / to_json_call
 DOUBLE_PRECISION = (
     Keyword("double", caseless=True) + Keyword("precision", caseless=True)("op")
-).addParseAction(lambda: {"double_precision": {}})
-NUMERIC = (
-    Keyword("numeric", caseless=True)("op") + _sizes
-).addParseAction(to_json_call)
+) / (lambda: {"double_precision": {}})
+NUMERIC = (Keyword("numeric", caseless=True)("op") + _sizes) / to_json_call
 
 
 DATE = Keyword("date", caseless=True)
@@ -410,12 +411,12 @@ time_functions = DATE | DATETIME | TIME | TIMESTAMP | TIMESTAMPTZ | TIMETZ
 # KNOWNN TIME TYPES
 _format = Optional((ansi_string | ansi_ident)("params"))
 
-DATE_TYPE = (DATE("op") + _format).addParseAction(to_json_call)
-DATETIME_TYPE = (DATETIME("op") + _format).addParseAction(to_json_call)
-TIME_TYPE = (TIME("op") + _format).addParseAction(to_json_call)
-TIMESTAMP_TYPE = (TIMESTAMP("op") + _format).addParseAction(to_json_call)
-TIMESTAMPTZ_TYPE = (TIMESTAMPTZ("op") + _format).addParseAction(to_json_call)
-TIMETZ_TYPE = (TIMETZ("op") + _format).addParseAction(to_json_call)
+DATE_TYPE = (DATE("op") + _format) / to_json_call
+DATETIME_TYPE = (DATETIME("op") + _format) / to_json_call
+TIME_TYPE = (TIME("op") + _format) / to_json_call
+TIMESTAMP_TYPE = (TIMESTAMP("op") + _format) / to_json_call
+TIMESTAMPTZ_TYPE = (TIMESTAMPTZ("op") + _format) / to_json_call
+TIMETZ_TYPE = (TIMETZ("op") + _format) / to_json_call
 
 known_types = MatchFirst([
     ARRAY,
@@ -451,3 +452,17 @@ known_types = MatchFirst([
     VARBINARY,
     TINYINT,
 ])
+
+CASTING = (Literal("::").suppress() + known_types("params")).set_parser_name("cast")
+KNOWN_OPS = [CASTING] + KNOWN_OPS
+unary_ops = {
+    NEG: RIGHT_ASSOC,
+    NOT: RIGHT_ASSOC,
+    BINARY_NOT: RIGHT_ASSOC,
+    CASTING: LEFT_ASSOC,
+}
+
+from mo_sql_parsing import utils
+
+utils.unary_ops = unary_ops
+del utils
