@@ -7,7 +7,7 @@
 # Contact: Kyle Lahnakoski (kyle@lahnakoski.com)
 #
 
-from mo_parsing.helpers import delimitedList, restOfLine
+from mo_parsing.helpers import delimited_list, restOfLine
 from mo_parsing.whitespaces import NO_WHITESPACE, Whitespace
 
 from mo_sql_parsing.keywords import *
@@ -16,7 +16,7 @@ from mo_sql_parsing.windows import window
 
 
 def combined_parser():
-    combined_ident = Combine(delimitedList(
+    combined_ident = Combine(delimited_list(
         ansi_ident | mysql_backtick_ident | sqlserver_ident | Word(IDENT_CHAR),
         separator=".",
         combine=True,
@@ -27,7 +27,7 @@ def combined_parser():
 
 def mysql_parser():
     mysql_string = ansi_string | mysql_doublequote_string
-    mysql_ident = Combine(delimitedList(
+    mysql_ident = Combine(delimited_list(
         mysql_backtick_ident | sqlserver_ident | Word(IDENT_CHAR),
         separator=".",
         combine=True,
@@ -52,27 +52,28 @@ def parser(literal_string, ident):
         case = (
             CASE
             + Group(ZeroOrMore(
-                (WHEN + expr("when") + THEN + expr("then")).addParseAction(to_when_call)
+                (WHEN + expr("when") + THEN + expr("then")) / to_when_call
             ))("case")
             + Optional(ELSE + expr("else"))
             + END
-        ).addParseAction(to_case_call)
+        ) / to_case_call
 
         # SWITCH
         switch = (
             CASE
             + expr("value")
             + Group(ZeroOrMore(
-                (WHEN + expr("when") + THEN + expr("then")).addParseAction(to_when_call)
+                (WHEN + expr("when") + THEN + expr("then")) / to_when_call
             ))("case")
             + Optional(ELSE + expr("else"))
             + END
-        ).addParseAction(to_switch_call)
+        ) / to_switch_call
 
         # CAST
-        cast = Group(
-            CAST("op") + LB + expr("params") + AS + known_types("params") + RB
-        ).addParseAction(to_json_call)
+        cast = (
+            Group(CAST("op") + LB + expr("params") + AS + known_types("params") + RB)
+            / to_json_call
+        )
 
         # TRIM
         trim = (
@@ -82,32 +83,30 @@ def parser(literal_string, ident):
                 + expr("chars")
                 + Optional(FROM + expr("from"))
                 + RB
-            )
-            .addParseAction(to_trim_call)
-            .set_parser_name("trim")
+            ).set_parser_name("trim")
+            / to_trim_call
         )
 
         _standard_time_intervals = MatchFirst([
-            Keyword(d, caseless=True).addParseAction(lambda t: durations[t[0].lower()])
+            Keyword(d, caseless=True) / (lambda t: durations[t[0].lower()])
             for d in durations.keys()
         ]).set_parser_name("duration")("params")
 
-        duration = (realNum | intNum)("params") + _standard_time_intervals
+        duration = (real_num | int_num)("params") + _standard_time_intervals
 
         interval = (
-            INTERVAL + ("'" + delimitedList(duration) + "'" | duration)
-        ).addParseAction(to_interval_call)
+            INTERVAL + ("'" + delimited_list(duration) + "'" | duration)
+        ) / to_interval_call
 
         timestamp = (
             time_functions("op")
             + (
                 literal_string("params")
                 | MatchFirst([
-                    Keyword(t, caseless=True).addParseAction(lambda t: t.lower())
-                    for t in times
+                    Keyword(t, caseless=True) / (lambda t: t.lower()) for t in times
                 ])("params")
             )
-        ).addParseAction(to_json_call)
+        ) / to_json_call
 
         extract = (
             Keyword("extract", caseless=True)("op")
@@ -116,99 +115,96 @@ def parser(literal_string, ident):
             + FROM
             + expr("params")
             + RB
-        ).addParseAction(to_json_call)
+        ) / to_json_call
 
-        namedColumn = Group(
+        named_column = Group(
             Group(expr)("value") + Optional(Optional(AS) + Group(var_name))("name")
         )
 
         distinct = (
-            DISTINCT("op") + delimitedList(namedColumn)("params")
-        ).addParseAction(to_json_call)
+            DISTINCT("op") + delimited_list(named_column)("params")
+        ) / to_json_call
 
         ordered_sql = Forward()
 
         call_function = (
             ident("op")
             + LB
-            + Optional(Group(ordered_sql) | delimitedList(Group(expr)))("params")
+            + Optional(Group(ordered_sql) | delimited_list(Group(expr)))("params")
             + Optional(
                 Keyword("ignore", caseless=True) + Keyword("nulls", caseless=True)
             )("ignore_nulls")
             + RB
-        ).addParseAction(to_json_call)
+        ) / to_json_call
 
         with NO_WHITESPACE:
 
             def scale(tokens):
                 return {"mul": [tokens[0], tokens[1]]}
 
-            scale_function = ((realNum | intNum) + call_function).addParseAction(scale)
-            scale_ident = ((realNum | intNum) + ident).addParseAction(scale)
+            scale_function = ((real_num | int_num) + call_function) / scale
+            scale_ident = ((real_num | int_num) + ident) / scale
 
         compound = (
-            NULL
-            | TRUE
-            | FALSE
-            | NOCASE
-            | interval
-            | timestamp
-            | extract
-            | case
-            | switch
-            | cast
-            | distinct
-            | trim
-            | (LB + Group(ordered_sql) + RB)
-            | (LB + Group(delimitedList(expr)).addParseAction(to_tuple_call) + RB)
-            | literal_string.set_parser_name("string")
-            | hexNum.set_parser_name("hex")
-            | scale_function
-            | scale_ident
-            | realNum.set_parser_name("float")
-            | intNum.set_parser_name("int")
-            | call_function
-            | known_types
-            | Combine(var_name + Optional(".*"))
+                NULL
+                | TRUE
+                | FALSE
+                | NOCASE
+                | interval
+                | timestamp
+                | extract
+                | case
+                | switch
+                | cast
+                | distinct
+                | trim
+                | (LB + Group(ordered_sql) + RB)
+                | (LB + Group(delimited_list(expr)) / to_tuple_call + RB)
+                | literal_string.set_parser_name("string")
+                | hex_num.set_parser_name("hex")
+                | scale_function
+                | scale_ident
+                | real_num.set_parser_name("float")
+                | int_num.set_parser_name("int")
+                | call_function
+                # | known_types
+                | Combine(var_name + Optional(".*"))
         )
 
-        sortColumn = expr("value").set_parser_name("sort1") + Optional(
+        sort_column = expr("value").set_parser_name("sort1") + Optional(
             DESC("sort") | ASC("sort")
         ) | expr("value").set_parser_name("sort2")
 
         expr << (
             (
                 Literal("*")
-                | infixNotation(
+                | infix_notation(
                     compound,
                     [
                         (
                             o,
                             1 if o in unary_ops else (3 if isinstance(o, tuple) else 2),
-                            RIGHT_ASSOC if o in unary_ops else LEFT_ASSOC,
+                            unary_ops[o] if o in unary_ops else LEFT_ASSOC,
                             to_json_operator,
                         )
                         for o in KNOWN_OPS
                     ],
                 ).set_parser_name("expression")
             )("value")
-            + Optional(window(expr, sortColumn))
-        ).addParseAction(to_expression_call)
+            + Optional(window(expr, sort_column))
+        ) / to_expression_call
 
         alias = (
-            (Group(var_name) + Optional(LB + delimitedList(ident("col")) + RB))("name")
-            .set_parser_name("alias")
-            .addParseAction(to_alias)
-        )
+            Group(var_name) + Optional(LB + delimited_list(ident("col")) + RB)
+        )("name").set_parser_name("alias") / to_alias
 
-        selectColumn = (
+        select_column = (
             Group(
                 Group(expr).set_parser_name("expression1")("value")
                 + Optional(Optional(AS) + alias)
                 | Literal("*")("value")
-            )
-            .set_parser_name("column")
-            .addParseAction(to_select_call)
+            ).set_parser_name("column")
+            / to_select_call
         )
 
         table_source = (
@@ -234,21 +230,36 @@ def parser(literal_string, ident):
             )("op")
             + Group(table_source)("join")
             + Optional((ON + expr("on")) | (USING + expr("using")))
-        ).addParseAction(to_join_call)
+        ) / to_join_call
+
+        selection = (
+            SELECT
+            + DISTINCT
+            + ON
+            + LB
+            + delimited_list(select_column)("distinct_on")
+            + RB
+            + delimited_list(select_column)("select")
+            | SELECT + DISTINCT + delimited_list(select_column)("select_distinct")
+            | (
+                SELECT
+                + Optional(
+                    TOP
+                    + expr("value")
+                    + Optional(Keyword("percent", caseless=True))("percent")
+                    + Optional(WITH + Keyword("ties", caseless=True))("ties")
+                )("top")
+                / to_top_clause
+                + delimited_list(select_column)("select")
+            )
+        )
 
         unordered_sql = Group(
-            SELECT
+            selection
             + Optional(
-                TOP
-                + expr("value")
-                + Optional(Keyword("percent", caseless=True))("percent")
-                + Optional(WITH + Keyword("ties", caseless=True))("ties")
-            )("top").addParseAction(to_top_clause)
-            + delimitedList(selectColumn)("select")
-            + Optional(
-                (FROM + delimitedList(Group(table_source)) + ZeroOrMore(join))("from")
+                (FROM + delimited_list(Group(table_source)) + ZeroOrMore(join))("from")
                 + Optional(WHERE + expr("where"))
-                + Optional(GROUP_BY + delimitedList(Group(namedColumn))("groupby"))
+                + Optional(GROUP_BY + delimited_list(Group(named_column))("groupby"))
                 + Optional(HAVING + expr("having"))
             )
         ).set_parser_name("unordered sql")
@@ -261,54 +272,53 @@ def parser(literal_string, ident):
                     + unordered_sql
                 )
             )("union")
-            + Optional(ORDER_BY + delimitedList(Group(sortColumn))("orderby"))
+            + Optional(ORDER_BY + delimited_list(Group(sort_column))("orderby"))
             + Optional(LIMIT + expr("limit"))
             + Optional(OFFSET + expr("offset"))
-        ).set_parser_name("ordered sql").addParseAction(to_union_call)
+        ).set_parser_name("ordered sql") / to_union_call
 
         statement = Forward()
         statement << (
             Optional(
                 WITH
-                + delimitedList(Group(
+                + delimited_list(Group(
                     var_name("name") + AS + LB + (statement | expr)("value") + RB
                 ))
             )("with")
             + Group(ordered_sql)("query")
-        ).addParseAction(to_statement)
+        ) / to_statement
 
         #####################################################################
         # CREATE TABLE
         #####################################################################
-        createStmt = Forward()
+        create_stmt = Forward()
 
         BigQuery_STRUCT = (
             Keyword("struct", caseless=True)("op")
             + Literal("<").suppress()
-            + delimitedList(column_definition)("params")
+            + delimited_list(column_definition)("params")
             + Literal(">").suppress()
-        ).addParseAction(to_json_call)
+        ) / to_json_call
 
         BigQuery_ARRAY = (
             Keyword("array", caseless=True)("op")
             + Literal("<").suppress()
-            + delimitedList(column_type)("params")
+            + delimited_list(column_type)("params")
             + Literal(">").suppress()
-        ).addParseAction(to_json_call)
+        ) / to_json_call
 
         column_type << (
             BigQuery_STRUCT
             | BigQuery_ARRAY
-            | Group(
-                ident("op") + Optional(LB + delimitedList(intNum)("params") + RB)
-            ).addParseAction(to_json_call)
+            | Group(ident("op") + Optional(LB + delimited_list(int_num)("params") + RB))
+            / to_json_call
         )
 
         column_def_references = (
             REFERENCES
             + var_name("table")
             + LB
-            + delimitedList(var_name)("columns")
+            + delimited_list(var_name)("columns")
             + RB
         )("references")
 
@@ -319,8 +329,8 @@ def parser(literal_string, ident):
         ).suppress() + expr("default")
 
         column_options = ZeroOrMore(Group(
-            (NOT + NULL).addParseAction(lambda: "not null")
-            | NULL.addParseAction(lambda t: "nullable")
+            (NOT + NULL) / (lambda: "not null")
+            | NULL / (lambda t: "nullable")
             | Keyword("unique", caseless=True)
             | Keyword("primary key", caseless=True)
             | column_def_references
@@ -329,7 +339,7 @@ def parser(literal_string, ident):
         )).set_parser_name("column_options")
 
         column_definition << Group(
-            var_name("name").addParseAction(lambda t: t[0].lower())
+            var_name("name") / (lambda t: t[0].lower())
             + column_type("type")
             + Optional(column_options)("option")
         ).set_parser_name("column_definition")
@@ -337,7 +347,7 @@ def parser(literal_string, ident):
         # MySQL's index_type := Using + ( "BTREE" | "HASH" )
         index_type = Optional(USING + ident("index_type"))
 
-        index_column_names = LB + delimitedList(var_name("columns")) + RB
+        index_column_names = LB + delimited_list(var_name("columns")) + RB
 
         column_def_foreign_key = FOREIGN_KEY + Optional(
             var_name("index_name") + index_column_names + column_def_references
@@ -372,15 +382,15 @@ def parser(literal_string, ident):
             column_definition("columns") | table_constraint_definition("constraint")
         )
 
-        createStmt << (
+        create_stmt << (
             CREATE_TABLE
             + (
                 var_name("name")
-                + Optional(LB + delimitedList(table_element) + RB)
+                + Optional(LB + delimited_list(table_element) + RB)
                 + Optional(
-                    AS.suppress() + infixNotation(statement, [])
+                    AS.suppress() + infix_notation(statement, [])
                 )("select_statement")
             )("create table")
         )
 
-        return (statement | createStmt).finalize()
+        return (statement | create_stmt).finalize()
