@@ -13,10 +13,11 @@ import ast
 
 from mo_dots import is_data, is_null, Data, from_data
 from mo_future import text, number_types, binary_type
-
 from mo_parsing import *
 from mo_parsing.utils import is_number, listwrap
 
+unary_ops = None
+
 
 class Call(object):
     __slots__ = ["op", "args", "kwargs"]
@@ -115,6 +116,8 @@ def to_json_operator(tokens):
     # ARRANGE INTO {op: params} FORMAT
     length = len(tokens.tokens)
     if length == 2:
+        if tokens.tokens[1].type.parser_name == "cast":
+            return Call("cast", list(tokens), {})
         # UNARY OPERATOR
         op = tokens.tokens[0].type.parser_name
         if op == "neg" and is_number(tokens[1]):
@@ -182,8 +185,8 @@ def to_tuple_call(tokens):
     if all(isinstance(r, number_types) for r in tokens):
         return [tokens]
     if all(
-            isinstance(r, number_types) or (is_data(r) and "literal" in r.keys())
-            for r in tokens
+        isinstance(r, number_types) or (is_data(r) and "literal" in r.keys())
+        for r in tokens
     ):
         candidate = {"literal": [r["literal"] if is_data(r) else r for r in tokens]}
         return candidate
@@ -213,12 +216,14 @@ binary_ops = {
     "<>": "neq",
     "not in": "nin",
     "in": "in",
-    "is not": "neq",
+    "is_not": "neq",
     "is": "eq",
     "similar_to": "similar_to",
     "like": "like",
     "rlike": "rlike",
+    "not like": "not_like",
     "not_like": "not_like",
+    "not rlike": "not_rlike",
     "not_rlike": "not_rlike",
     "not_simlilar_to": "not_similar_to",
     "or": "or",
@@ -257,7 +262,7 @@ def to_json_call(tokens):
         tokens.start,
         tokens.end,
         [Call(op, params, {"ignore_nulls": ignore_nulls})],
-        tokens.failures
+        tokens.failures,
     )
 
 
@@ -319,7 +324,11 @@ def to_expression_call(tokens):
         return
 
     expr = ParseResults(
-        tokens.type, tokens.start, tokens.end, listwrap(tokens["value"]), tokens.failures
+        tokens.type,
+        tokens.start,
+        tokens.end,
+        listwrap(tokens["value"]),
+        tokens.failures,
     )
     return expr
 
@@ -356,8 +365,9 @@ def to_select_call(tokens):
         return ["*"]
 
     if value["over"] or value["within"]:
-        output = ParseResults(tokens.type, tokens.start, tokens.end, value.tokens, tokens.failures
-                              )
+        output = ParseResults(
+            tokens.type, tokens.start, tokens.end, value.tokens, tokens.failures
+        )
         output["name"] = tokens["name"]
         return output
     else:
@@ -421,10 +431,9 @@ def to_string(tokens):
 
 
 # NUMBERS
-realNum = (
-    Regex(r"[+-]?(\d+\.\d*|\.\d+)([eE][+-]?\d+)?")
-        .set_parser_name("float")
-        .addParseAction(lambda t: float(t[0]))
+real_num = (
+    Regex(r"[+-]?(\d+\.\d*|\.\d+)([eE][+-]?\d+)?").set_parser_name("float")
+    / (lambda t: float(t[0]))
 )
 
 
@@ -435,20 +444,14 @@ def parse_int(tokens):
         return int(tokens[0])
 
 
-intNum = (
-    Regex(r"[+-]?\d+([eE]\+?\d+)?").set_parser_name("int").addParseAction(parse_int)
-)
-hexNum = (
-    Regex(r"0x[0-9a-fA-F]+")
-        .set_parser_name("hex")
-        .addParseAction(lambda t: {"hex": t[0][2:]})
-)
+int_num = Regex(r"[+-]?\d+([eE]\+?\d+)?").set_parser_name("int") / parse_int
+hex_num = Regex(r"0x[0-9a-fA-F]+").set_parser_name("hex") / (lambda t: {"hex": t[0][2:]})
 
 # STRINGS
-ansi_string = Regex(r"\'(\'\'|[^'])*\'").addParseAction(to_string)
-mysql_doublequote_string = Regex(r'\"(\"\"|[^"])*\"').addParseAction(to_string)
+ansi_string = Regex(r"\'(\'\'|[^'])*\'") / to_string
+mysql_doublequote_string = Regex(r'\"(\"\"|[^"])*\"') / to_string
 
 # BASIC IDENTIFIERS
-ansi_ident = Regex(r'\"(\"\"|[^"])*\"').addParseAction(unquote)
-mysql_backtick_ident = Regex(r"\`(\`\`|[^`])*\`").addParseAction(unquote)
-sqlserver_ident = Regex(r"\[(\]\]|[^\]])*\]").addParseAction(unquote)
+ansi_ident = Regex(r'\"(\"\"|[^"])*\"') / unquote
+mysql_backtick_ident = Regex(r"\`(\`\`|[^`])*\`") / unquote
+sqlserver_ident = Regex(r"\[(\]\]|[^\]])*\]") / unquote
